"use strict";
/*---------------------------------------------------------
 * Copyright (C) Microsoft Corporation. All rights reserved.
 *--------------------------------------------------------*/
Object.defineProperty(exports, "__esModule", { value: true });
const fs = require("fs");
const uuid = require("node-uuid");
const vscode = require("vscode");
const constants_1 = require("./constants");
const status_1 = require("./status");
const editorService_1 = require("./services/editorService");
const symbolsService_1 = require("./services/symbolsService");
const symbolsDownloader_1 = require("./services/symbolsDownloader");
const launchService_1 = require("./services/launchService");
const contentService_1 = require("./services/contentService");
const bootstrapService_1 = require("./services/bootstrapService");
const downloadSourceService_1 = require("./services/downloadSourceService");
const openPageDesignerService_1 = require("./services/openPageDesignerService");
const buildService_1 = require("./services/buildService");
const referenceContentProvider_1 = require("./services/referenceContentProvider");
var extension = null;
function activate(context) {
    updateUserIdIfNeeded(context);
    extension = new AlExtension();
    extension.activateServices(context);
    vscode.commands.executeCommand('setContext', constants_1.AlExtensionContextName, true);
}
exports.activate = activate;
function deactivate() {
    vscode.commands.executeCommand('setContext', constants_1.AlExtensionContextName, false);
}
exports.deactivate = deactivate;
class AlExtension {
    constructor() {
        this.services = [];
    }
    activateServices(context) {
        let launchService = new launchService_1.LaunchService(context);
        this.services.push(launchService);
        let editorService = new editorService_1.EditorService(context, launchService);
        this.services.push(editorService);
        let languageServerClient = editorService.tryStartLanguageServer();
        let preferredChannel = null;
        if (languageServerClient) {
            preferredChannel = languageServerClient.outputChannel;
        }
        status_1.initializeOuputChannel(preferredChannel);
        let symbolsDownloader = new symbolsDownloader_1.SymbolsDownloader(languageServerClient);
        let symbolsService = new symbolsService_1.SymbolsService(context, launchService, symbolsDownloader, editorService);
        this.services.push(symbolsService);
        let contentService = new contentService_1.ContentService(context, languageServerClient);
        this.services.push(contentService);
        let bootstrapService = new bootstrapService_1.BootstrapService(context, launchService, symbolsService, contentService);
        this.services.push(bootstrapService);
        let downloadSourceService = new downloadSourceService_1.DownloadSourceService(context, launchService, languageServerClient);
        this.services.push(downloadSourceService);
        let buildService = new buildService_1.BuildService(context, languageServerClient, symbolsService, launchService, editorService);
        this.services.push(buildService);
        let openPageService = new openPageDesignerService_1.OpenPageDesignerService(context, launchService, languageServerClient, buildService);
        this.services.push(openPageService);
        let referenceContentProvider = new referenceContentProvider_1.ReferenceContentProvider(context, languageServerClient);
        this.services.push(referenceContentProvider);
        this.services.forEach(x => x.activate());
    }
}
exports.AlExtension = AlExtension;
/**
 * Synchronizes the anonymous user ID between a local file and global workspace state, if necessary.
 * The global workspace state is preserved over vscode updates, so it is more durable than local extension file.
 * The user ID should then be read by the language server or debug service when sending telemetry.
 */
function updateUserIdIfNeeded(context) {
    try {
        var path = context.asAbsolutePath(constants_1.UserIdFile);
        let userId = context.globalState.get(constants_1.UserIdStateKey);
        if (fs.existsSync(path)) {
            if (!userId) {
                userId = fs.readFileSync(path, 'utf-8');
                context.globalState.update(constants_1.UserIdStateKey, userId);
            }
        }
        else {
            if (!userId) {
                userId = uuid.v4();
                context.globalState.update(constants_1.UserIdStateKey, userId);
            }
            fs.writeFileSync(path, userId, { encoding: 'utf-8' });
        }
    }
    catch (e) {
        //ignore
    }
}
//# sourceMappingURL=extension.js.map