"use strict";
/*---------------------------------------------------------
 * Copyright (C) Microsoft Corporation. All rights reserved.
 *--------------------------------------------------------*/
Object.defineProperty(exports, "__esModule", { value: true });
const vscode = require("vscode");
const path = require("path");
const fs = require("fs");
const extensionService_1 = require("./extensionService");
const status_1 = require("../status");
const osHelpers_1 = require("../osHelpers");
const resources_1 = require("../resources");
const constants_1 = require("../constants");
const serverProxy_1 = require("./serverProxy");
class BuildService extends extensionService_1.ExtensionService {
    constructor(context, languageServerClient, symbolsService, launchService, editorService) {
        super(context);
        this.languageServerClient = languageServerClient;
        this.symbolsService = symbolsService;
        this.launchService = launchService;
        this.editorService = editorService;
        this.serverProxy = new serverProxy_1.ServerProxy(this.languageServerClient);
    }
    activate() {
        let extensionRoot = this.context.asAbsolutePath('');
        this.registerFolderCommand('al.package', () => {
            this.packageContainer(extensionRoot)
                .catch(() => {
                //ignore
            });
        });
        this.registerFolderCommand('al.publish', () => {
            this.publishContainer(extensionRoot)
                .catch(() => {
                //ignore
            });
        });
        this.registerFolderCommand('al.publishNoDebug', () => {
            this.publishContainer(extensionRoot, true)
                .catch(() => {
                //ignore
            });
        });
        this.registerFolderCommand('al.clearCredentialsCache', () => {
            this.clearCredentialsCache(extensionRoot)
                .catch(() => {
                //ignore
            });
        });
        this.context.workspaceState.update(constants_1.SymbolsCheckedStateKey, false);
    }
    packageContainer(extensionRoot) {
        status_1.outputChannel.show();
        status_1.outputChannel.clear();
        return this.editorService.saveAllDocuments()
            .then(() => this.downloadSymbolsOnce())
            .then(() => this.performBuild());
    }
    publishContainer(extensionRoot, noDebug) {
        status_1.outputChannel.show();
        status_1.outputChannel.clear();
        return this.packageContainer(extensionRoot)
            .then(() => this.ensureAuthenticated())
            .then(() => {
            //VSCode not always switches to the debug console, where all the useful message are.
            //Therefore, debug console is activated after a certain delay.
            //If this is the first time debugging is started, do not show the console since debugger selection list has focus.
            if (fs.existsSync(path.join(vscode.workspace.rootPath, '.vscode', 'launch.json'))) {
                setTimeout(() => vscode.commands.executeCommand('workbench.debug.action.focusRepl'), 1000);
            }
        })
            .then(() => {
            vscode.commands.executeCommand(noDebug ? 'workbench.action.debug.run' : 'workbench.action.debug.start');
        })
            .catch(() => {
            //ignored
        });
    }
    downloadSymbolsOnce() {
        let symbolsPromise = null;
        if (!this.context.workspaceState.get(constants_1.SymbolsCheckedStateKey)) {
            symbolsPromise = this.symbolsService.checkSymbols()
                .then(result => {
                if (!result) {
                    return this.symbolsService.downloadSymbols(true);
                }
                return null;
            })
                .catch(() => {
                //ignore symbol errors to get compilation errors in the console as well
            });
        }
        else {
            symbolsPromise = Promise.resolve();
        }
        return symbolsPromise;
    }
    performBuild() {
        return new Promise((resolve, reject) => {
            let workspaceRoot = vscode.workspace.rootPath;
            let args = ['-project:"' + workspaceRoot + '"'];
            this.languageServerClient.sendRequest(constants_1.AlCreatePackageRequest, { projectDir: workspaceRoot, args })
                .then((response) => {
                status_1.outputChannel.appendLine(response.success ? resources_1.default.packageCreatedMessage : resources_1.default.packageNotCreatedError);
                if (response.success) {
                    resolve();
                    this.context.workspaceState.update(constants_1.SymbolsCheckedStateKey, true);
                }
                else {
                    reject();
                }
            }, reject);
        });
    }
    /**
     * Ensures that the user is authenticated in case if publishing is the first action which requires server communication.
     * Note that if multiple configurations are specified, the extension code cannot know which one will picked up by VSCode,
     * therefore in this case it is assumed that the user is already authenticated (from the previous actions).
     */
    ensureAuthenticated() {
        var configurations = this.launchService.getConfigurations();
        if (configurations.length !== 1) {
            return Promise.resolve();
        }
        return this.serverProxy.ensureAuthenticated(configurations[0]);
    }
    clearCredentialsCache(extensionRoot) {
        let exePath = path.join(extensionRoot, 'bin/webdeploy.exe');
        let args = ['clearCache'];
        status_1.outputChannel.show();
        return osHelpers_1.spawn(exePath, args)
            .catch(e => {
            status_1.outputChannel.appendLine(resources_1.default.credentialsCacheNotClearedError);
            throw e;
        });
    }
}
exports.BuildService = BuildService;
//# sourceMappingURL=buildService.js.map