"use strict";
/*---------------------------------------------------------
 * Copyright (C) Microsoft Corporation. All rights reserved.
 *--------------------------------------------------------*/
Object.defineProperty(exports, "__esModule", { value: true });
const path = require("path");
const uuid = require("node-uuid");
const vscode = require("vscode");
const constants_1 = require("../constants");
const fsHelpers_1 = require("../fsHelpers");
const resources_1 = require("../resources");
const vscodeHelpers_1 = require("../vscodeHelpers");
const extensionService_1 = require("./extensionService");
/**
 * The class for generating artifacts such as app.json and other content files.
 */
class ContentService extends extensionService_1.ExtensionService {
    constructor(context, languageServerClient) {
        super(context);
        this.languageServerClient = languageServerClient;
        if (this.languageServerClient) {
            this.languageServerClient.onReady()
                .then(() => this.languageServerClient.onNotification(constants_1.AlManifestMissingEvent, () => this.onManifestMissingEvent()));
        }
    }
    activate() {
        var disposable = vscode.commands.registerCommand('al.generateManifest', () => this.generateAppJson(vscode.workspace.rootPath, true));
        this.registerForDisposal(disposable);
    }
    onManifestMissingEvent() {
        vscode.window.showWarningMessage(resources_1.default.manifestMissingWarning, resources_1.default.generateManifestAction)
            .then(choice => {
            if (choice === resources_1.default.generateManifestAction) {
                this.generateAppJson(vscode.workspace.rootPath, true);
            }
        });
    }
    generateAppJson(rootPath, show = false) {
        var appJson = path.join(rootPath, constants_1.AlProjectFileName);
        let result = fsHelpers_1.existsFileAsync(appJson)
            .then(exists => {
            if (exists) {
                return appJson;
            }
            var json = {
                "id": uuid.v4(),
                "name": path.basename(rootPath),
                "publisher": "Default publisher",
                "brief": "",
                "description": "",
                "version": "1.0.0.0",
                "compatibilityId": "1.0.0.0",
                "privacyStatement": "",
                "EULA": "",
                "help": "",
                "url": "",
                "logo": "",
                "capabilities": [],
                "dependencies": [],
                "screenshots": [],
                "platform": "11.0.0.0",
                "application": "11.0.0.0",
                "idRange": {
                    "from": 50100,
                    "to": 50149
                }
            };
            return fsHelpers_1.writeTextFileAsync(appJson, JSON.stringify(json, null, '  '))
                .then(() => appJson);
        });
        if (show) {
            result = result.then(appJson => vscodeHelpers_1.openTextDocument(appJson));
        }
        return result;
    }
    generateDefaultAlFile(rootPath) {
        //samples could be downloaded from our github here
        var filePath = path.join(rootPath, constants_1.DefaultAlFileName);
        return fsHelpers_1.existsFileAsync(filePath)
            .then(exists => {
            if (exists) {
                return filePath;
            }
            var contents = `// Welcome to your new AL extension.
// Remember that object names and IDs should be unique across all extensions.
// AL snippets start with t*, like tpageext - give them a try and happy coding!

pageextension 50100 CustomerListExt extends "Customer List"
{
    trigger OnOpenPage();
    begin
        Message('App published: Hello world');
    end;
}`;
            return fsHelpers_1.writeTextFileAsync(filePath, contents)
                .then(() => filePath);
        });
    }
}
exports.ContentService = ContentService;
//# sourceMappingURL=contentService.js.map