"use strict";
/*---------------------------------------------------------
 * Copyright (C) Microsoft Corporation. All rights reserved.
 *--------------------------------------------------------*/
Object.defineProperty(exports, "__esModule", { value: true });
const vscode = require("vscode");
const util = require("util");
const vscode_languageclient_1 = require("vscode-languageclient");
const constants_1 = require("../constants");
const resources_1 = require("../resources");
const status_1 = require("../status");
const extensionService_1 = require("./extensionService");
/**
 * When configuration documents are saved, vscode fires onDidChangeConfiguration which serves as an indicator
 * that the caller can continue executing.
 * However, a safety measure is also put to ensure that calling saveAllDocuments always resolves a promise.
 * If no event is raised within certain time, saveAllDocuments still resolves a promise.
 */
const SafetyConfigurationUpdateTimeout = 2000;
/**
 * The service responsible for starting the language service, packaging and publishing commands.
 */
class EditorService extends extensionService_1.ExtensionService {
    constructor(context, launchService) {
        super(context);
        this.launchService = launchService;
        this.languageServerClient = null;
        /**
         * A deferred promise resolution callback for saving all documents in case a configuration is dirty and needs to be updated.
         */
        this.deferredResolveSaveAll = null;
        this.safetyConfigurationTimer = null;
        this.onConfigurationChanged = () => {
            if (this.safetyConfigurationTimer) {
                clearTimeout(this.safetyConfigurationTimer);
                this.safetyConfigurationTimer = null;
            }
            if (this.deferredResolveSaveAll) {
                this.deferredResolveSaveAll();
                this.deferredResolveSaveAll = null;
            }
        };
        this.registerForDisposal(vscode.workspace.onDidChangeConfiguration(this.onConfigurationChanged));
    }
    tryStartLanguageServer() {
        if (!vscode.workspace.rootPath) {
            return null;
        }
        let args = [];
        let alConfig = vscode.workspace.getConfiguration(constants_1.AlLanguageId);
        let editorServicesLogLevel = alConfig.get('editorServicesLogLevel');
        if (editorServicesLogLevel) {
            args.push('/logLevel:' + editorServicesLogLevel);
        }
        if (alConfig.get("debug")) {
            args.push("/waitForDebugger");
        }
        let serverPath = this.getServerPath();
        let serverOptions = {
            run: {
                command: serverPath,
                args: args
            },
            debug: {
                command: serverPath,
                args: args
            }
        };
        let clientOptions = {
            documentSelector: [constants_1.AlLanguageId, 'json'],
            synchronize: {
                configurationSection: constants_1.AlLanguageId,
                fileEvents: vscode.workspace.createFileSystemWatcher('**/*')
            }
        };
        let languageServerClient = new vscode_languageclient_1.LanguageClient(constants_1.AlLanguageClientName, serverOptions, clientOptions);
        languageServerClient.onReady().then(() => { }, (reason) => vscode.window.showErrorMessage(util.format(resources_1.default.languageClientStartupError, reason)));
        var disposable = languageServerClient.start();
        this.registerForDisposal(disposable);
        this.languageServerClient = languageServerClient;
        return this.languageServerClient;
    }
    /**
     * Saves all dirty documents and in case a launch.json is dirty, waits for vscode configuration service to be updated.
     * If the configuration file is dirty, it takes some time after saving until vscode api return the new data from the file.
     * Therefore in this case, a promise resolution callback is stored locally and invoked later when vscode signals that configuration has been changed.
     */
    saveAllDocuments() {
        return new Promise((resolve, reject) => {
            let configurationDirty = this.launchService.isDirty();
            vscode.workspace.saveAll(true).then(saved => {
                if (!saved) {
                    status_1.outputChannel.appendLine(resources_1.default.notAllFilesSavedError);
                    return reject();
                }
                if (configurationDirty) {
                    this.deferredResolveSaveAll = resolve;
                    this.safetyConfigurationTimer = setTimeout(this.onConfigurationChanged, SafetyConfigurationUpdateTimeout);
                }
                else {
                    resolve();
                }
            }, reject);
        });
    }
}
exports.EditorService = EditorService;
//# sourceMappingURL=editorService.js.map