"use strict";
/*---------------------------------------------------------
 * Copyright (C) Microsoft Corporation. All rights reserved.
 *--------------------------------------------------------*/
Object.defineProperty(exports, "__esModule", { value: true });
const vscode = require("vscode");
const path = require("path");
const fs = require("fs");
const util = require("util");
const constants_1 = require("../constants");
const resources_1 = require("../resources");
const fsHelpers_1 = require("../fsHelpers");
const vscodeHelpers_1 = require("../vscodeHelpers");
const extensionService_1 = require("./extensionService");
const FilePath = '.vscode/launch.json';
const LaunchFileVersion = '0.2.0';
const ConfigHeader = {
    "type": constants_1.AlLanguageId,
    "request": "launch"
};
const CloudConfig = {
    name: "Microsoft cloud sandbox",
    startupObjectId: 22
};
const LocalConfig = {
    name: "Your own server",
    server: "http://localhost",
    serverInstance: "nav",
    authentication: "UserPassword",
    startupObjectId: 22
};
/**
 * The service responsible for working with launch.json file - generating it, reading configurations, etc.
 */
class LaunchService extends extensionService_1.ExtensionService {
    constructor(context) {
        super(context);
    }
    activate() {
        let disposable = vscode.debug.registerDebugConfigurationProvider(constants_1.AlLanguageId, new AlDebugConfigurationProvider(this));
        this.registerForDisposal(disposable);
        disposable = vscode.commands.registerCommand('al.resolveDebugCommand', () => this.resolveDebugCommand());
        this.registerForDisposal(disposable);
    }
    getLaunchConfiguration() {
        let configurations = this.getConfigurations();
        if (configurations.length === 0) {
            this.generateLaunchJson()
                .then(success => {
                if (success) {
                    this.openLaunchJson();
                }
            });
            return Promise.reject(new Error(resources_1.default.noLaunchFileWarning));
        }
        let configurationChosen = null;
        if (configurations.length === 1) {
            configurationChosen = Promise.resolve(configurations[0]);
        }
        else {
            configurationChosen = this.chooseLaunchConfiguration(configurations);
        }
        return configurationChosen;
    }
    chooseLaunchConfiguration(configurations) {
        return new Promise((resolve, reject) => {
            var items = configurations.map(x => ({
                label: x.name,
                detail: this.formatLaunchConfiguration(x),
                description: ''
            }));
            var options = {
                placeHolder: resources_1.default.chooseServerPrompt,
                matchOnDetail: true
            };
            vscode.window.showQuickPick(items, options).then(item => {
                if (item) {
                    var i = items.indexOf(item);
                    if (i !== -1) {
                        resolve(configurations[i]);
                    }
                    else {
                        reject();
                    }
                }
                else {
                    reject();
                }
            });
        });
    }
    isConfigured() {
        if (!vscode.workspace.rootPath) {
            return false;
        }
        return fs.existsSync(this.getFilePath());
    }
    isDirty() {
        let filePath = this.getFilePath();
        for (let i = 0; i < vscode.workspace.textDocuments.length; i++) {
            let doc = vscode.workspace.textDocuments[i];
            if (doc.isDirty && doc.fileName === filePath) {
                return true;
            }
        }
        return false;
    }
    getConfigurations() {
        if (!this.isConfigured()) {
            return [];
        }
        var config = vscode.workspace.getConfiguration('launch');
        var configurations = config.get('configurations');
        if (!Array.isArray(configurations)) {
            vscode.window.showErrorMessage(util.format(resources_1.default.jsonMalformedError, FilePath));
            return [];
        }
        return configurations.filter(x => x.type === constants_1.AlLanguageId);
    }
    generateLaunchJson() {
        var filePath = this.getFilePath();
        return fsHelpers_1.existsFileAsync(filePath)
            .then(exists => {
            if (exists) {
                return filePath;
            }
            var dir = path.dirname(filePath);
            if (!fs.existsSync(dir)) {
                try {
                    fs.mkdirSync(dir);
                }
                catch (e) {
                    return null;
                }
            }
            return this.getInitialFileContent()
                .then(config => vscode.workspace.getConfiguration().update('launch', config))
                .then(() => filePath);
        });
    }
    openLaunchJson() {
        return vscodeHelpers_1.openTextDocument(this.getFilePath());
    }
    formatLaunchConfiguration(config) {
        if (config.server) {
            return util.format(resources_1.default.serverFormat, config.server, config.serverInstance, config.tenant || 'default');
        }
        return resources_1.default.cloudServer;
    }
    getFilePath() {
        return path.join(vscode.workspace.rootPath, FilePath);
    }
    resolveDebugCommand() {
        var args = ["/startDebugging"];
        var alConfig = vscode.workspace.getConfiguration(constants_1.AlLanguageId);
        var env = alConfig.get("env");
        if (env) {
            args.push("/env:" + env);
        }
        if (alConfig.get("debug")) {
            args.push("/waitForDebugger");
        }
        let editorServicesLogLevel = alConfig.get('editorServicesLogLevel');
        if (editorServicesLogLevel) {
            args.push('/logLevel:' + editorServicesLogLevel);
        }
        args.push("/projectRoot:" + vscode.workspace.rootPath);
        let serverPath = this.getServerPath();
        return {
            command: serverPath,
            args
        };
    }
    getInitialConfigurations() {
        return this.chooseLaunchConfiguration([CloudConfig, LocalConfig])
            .catch(() => LocalConfig)
            .then(config => [Object.assign({}, ConfigHeader, config)]);
    }
    getInitialFileContent() {
        return this.getInitialConfigurations()
            .then(configs => {
            var initialConfigurations = {
                version: LaunchFileVersion,
                configurations: configs
            };
            return initialConfigurations;
        });
    }
}
exports.LaunchService = LaunchService;
class AlDebugConfigurationProvider {
    constructor(launchService) {
        this.launchService = launchService;
    }
    provideDebugConfigurations(folder, token) {
        return this.launchService.getInitialConfigurations();
    }
}
//# sourceMappingURL=launchService.js.map