"use strict";
/*---------------------------------------------------------
 * Copyright (C) Microsoft Corporation. All rights reserved.
 *--------------------------------------------------------*/
Object.defineProperty(exports, "__esModule", { value: true });
const userPasswordAuth_1 = require("./userPasswordAuth");
const constants_1 = require("../constants");
/**
 * The class for communicating with the web server via the language server.
 * The proxy handles the UserPassword authentication scenarios by examining error messages and repeating the requests with the new credentials.
 */
class ServerProxy {
    constructor(languageServerClient) {
        this.languageServerClient = languageServerClient;
        this.userPasswordAuth = new userPasswordAuth_1.UserPasswordAuth(languageServerClient);
    }
    /**
     * Ensures that the user is authenticated, which means that:
     * - Either AAD/Windows authentication is used.
     * - UserPassword authentication is used and the user has entered the credentials.
     * @param launchConfiguration The launch configuration.
     */
    ensureAuthenticated(launchConfiguration) {
        if (launchConfiguration.authentication !== "UserPassword") {
            return Promise.resolve();
        }
        return new Promise((resolve, reject) => {
            this.languageServerClient.sendRequest(constants_1.AlCheckAuthenticatedRequest, { launchConfiguration })
                .then(result => {
                if (result.authenticated) {
                    resolve();
                }
                else {
                    this.userPasswordAuth.authenticate(launchConfiguration).then(resolve, reject);
                }
            }, reject);
        });
    }
    /**
     * Sends the request and handles the authentication scenarios by re-sending the requests with the new credentials if necessary.
     *
     * @param launchConfiguration The launch configuration.
     * @param request The request method.
     * @param args The request parameters.
     */
    sendRequest(launchConfiguration, request, args) {
        var params = Object.assign({ launchConfiguration }, args);
        return new Promise((resolve, reject) => {
            this.sendRequestInternal(launchConfiguration, request, params, resolve, reject);
        });
    }
    sendRequestInternal(launchConfiguration, request, params, resolve, reject) {
        return this.languageServerClient.sendRequest(request, params)
            .then(resolve, e => this.onError(e, launchConfiguration, request, params, resolve, reject));
    }
    onError(error, launchConfiguration, request, params, resolve, reject) {
        if (error.data === ServerErrorCode.Unauthorized) {
            if (launchConfiguration.authentication !== "UserPassword") {
                reject(error);
                return;
            }
            this.userPasswordAuth.authenticate(launchConfiguration)
                .then(() => this.sendRequestInternal(launchConfiguration, request, params, resolve, reject));
            return;
        }
        reject(error);
    }
}
exports.ServerProxy = ServerProxy;
var ServerErrorCode;
(function (ServerErrorCode) {
    ServerErrorCode[ServerErrorCode["Unauthorized"] = 401] = "Unauthorized";
})(ServerErrorCode || (ServerErrorCode = {}));
//# sourceMappingURL=serverProxy.js.map